print("import: 'fluiddoc'")
import fluiddoc

print("import: 'fluiddyn'")
import fluiddyn

print("import: 'fluiddyn.calcul'")
import fluiddyn.calcul

print("import: 'fluiddyn.calcul.test'")
import fluiddyn.calcul.test

print("import: 'fluiddyn.clusters'")
import fluiddyn.clusters

print("import: 'fluiddyn.clusters.test'")
import fluiddyn.clusters.test

print("import: 'fluiddyn.io'")
import fluiddyn.io

print("import: 'fluiddyn.io.test'")
import fluiddyn.io.test

print("import: 'fluiddyn.output'")
import fluiddyn.output

print("import: 'fluiddyn.output.test'")
import fluiddyn.output.test

print("import: 'fluiddyn.util'")
import fluiddyn.util

print("import: 'fluiddyn.util.matlab2py'")
import fluiddyn.util.matlab2py

print("import: 'fluiddyn.util.matlab2py.test'")
import fluiddyn.util.matlab2py.test

print("import: 'fluiddyn.util.test'")
import fluiddyn.util.test

