"""
Constants (:mod:`fluiddyn.util.constants`)
==========================================


g = 9.81 m/s^2

nu_pure_water = 1.e-6 m^2/s

rho0 = 1. kg/m^3

rho_pure_water = 0.998 kg/m^3


"""

from __future__ import division, print_function

import numpy as np

g = 9.81  # (m/s^2)
nu_pure_water = 1.0e-6  # (m^2/s)
rho0 = 1.0  # (kg/m^3)
rho_pure_water = 0.998  # (kg/m^3)

del (division, print_function, np)
