



fluidinfo --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
fluidnbstripout --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
fluiddocset --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
fluiddump --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
fluidmat2py --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
fluidcluster-help --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
